import { MatDialogRef, MAT_DIALOG_DATA } from "@angular/material";
import { Inject, Component } from "@angular/core";
import { I18nService } from "../../services/internationalisation/internationalisation.service";
import { FormGroup, FormBuilder, Validators } from "@angular/forms";
import { ServiceFactory } from "../../services/service-factory";

@Component({
    selector: "dialog-load-session",
    templateUrl: "dialog-load-session.component.html",
    styleUrls: ["dialog-load-session.component.scss"]
})
export class DialogLoadSessionComponent {

    public calculators: any[] = [];

    public file: any;

    public loadSessionForm: FormGroup;

    constructor(
      public dialogRef: MatDialogRef<DialogLoadSessionComponent>,
      private intlService: I18nService,
      private fb: FormBuilder,
      @Inject(MAT_DIALOG_DATA) public data: any
    ) {
      this.loadSessionForm = this.fb.group({
        file: [null, Validators.required]
      });
    }

    public selectAll() {
      for (const c of this.calculators) {
        c.selected = true;
      }
    }

    public selectNone() {
      for (const c of this.calculators) {
        c.selected = false;
      }
    }

    public onFileSelected(event: any) {
      if (event.target.files && event.target.files.length) {
        this.file = event.target.files[0];

        const formService = ServiceFactory.instance.formulaireService;
        formService.calculatorInfosFromSessionFile(this.file).then(
          calcInfos => {
            this.calculators = calcInfos;
            for (const n of this.calculators) {
                n.selected = true;
            }
          }
        );
      }
    }

    public loadSession() {
      this.dialogRef.close({
        calculators: this.calculators,
        file: this.file
      });
    }

    public get atLeastOneCheckboxSelected() {
      let ok = false;
      for (const c of this.calculators) {
        ok = (ok || c.selected);
      }
      return ok;
    }

    public get uitextLoad() {
      return this.intlService.localizeText("INFO_OPTION_LOAD");
    }

    public get uitextCancel() {
      return this.intlService.localizeText("INFO_OPTION_CANCEL");
    }

    public get uitextAll() {
      return this.intlService.localizeText("INFO_OPTION_ALL");
    }

    public get uitextNone() {
      return this.intlService.localizeText("INFO_OPTION_NONE");
    }

    public get uitextLoadSessionFilename() {
      return this.intlService.localizeText("INFO_DIALOG_LOAD_SESSION_FILENAME");
    }

    public get uitextLoadSessionTitle() {
      return this.intlService.localizeText("INFO_DIALOG_LOAD_SESSION_TITLE");
    }
}
