import { Component, ViewChild, ElementRef } from "@angular/core";
import { VarResults } from "../../results/var-results";
import { ApplicationSetupService } from "../../services/app-setup/app-setup.service";
import { ResultElement } from "jalhyd";
import { I18nService } from "../../services/internationalisation/internationalisation.service";
import * as XLSX from "xlsx";

@Component({
    selector: "var-results",
    templateUrl: "./var-results.component.html",
    styleUrls: [
        "./var-results.component.scss"
    ]
})
export class VarResultsComponent {

    /** résultats non mis en forme */
    private _varResults: VarResults;

    /** résultats mis en forme */
    private _results: any[];

    /** entêtes des colonnes (param à varier, à calculer + extraResults) */
    private _headers: string[];

    /** tracks the fullscreen state */
    public get isFullscreen() {
        return (document["fullscreenElement"] !== null);
    }

    @ViewChild("tableContainer") table: ElementRef;

    constructor(
        private appSetupService: ApplicationSetupService,
        private intlService: I18nService
    ) { }

    public set results(r: VarResults) {
        this._varResults = r;

        this._results = [];
        this._headers = [];

        if (this._varResults) {
            const nDigits = this.appSetupService.displayDigits;
            let i = 0;
            for (const x of this._varResults.variatedParameter.valuesIterator) {
                const pval = x.toFixed(nDigits);
                this._results.push({ "param": pval, "result": this._varResults.resultElements[i] });
                i++;
            }

            this._headers.push(this._varResults.variableParamHeader);
            this._headers.push(this._varResults.calculatedParameterHeader);
            this._headers = this._headers.concat(this._varResults.extraResultHeaders);
        }
    }

    public get hasResults(): boolean {
        return this._varResults && this._varResults.hasResults;
    }

    public get headers() {
        return this._headers;
    }

    /**
     * Returns a combination of and results and extraResults for mat-table
     */
    public get dataSet() {
        const data = [];
        const nDigits = this.appSetupService.displayDigits;
        if (this._results) {
            for (let i = 0; i < this._results.length; i++) {
                const r = this._results[i];
                const re: ResultElement = this._varResults.resultElements[i];

                // for each computation step, build ordered list of : variable param value; result; extra results

                // 1. variable param value
                const list = [ r.param ];

                // 2. result
                list.push(re.vCalc.toFixed(nDigits));

                // 3. extra results
                for (const erk of this._varResults.extraResultKeys) {
                    const er = re.getExtraResult(erk);
                    if (er !== undefined) {
                        list.push(this.intlService.formatResult(erk, er));
                    } else {
                        list.push(er); // keep list ordered
                    }

                }
                data.push(list);
            }
        }
        return data;
    }

    public setFullscreen(element) {
        element.requestFullscreen();
    }

    public exitFullscreen() {
        document.exitFullscreen();
    }

    public exportAsSpreadsheet() {
        // automagic <table> extraction
        const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(this.table.nativeElement);
        const wb: XLSX.WorkBook = XLSX.utils.book_new();
        XLSX.utils.book_append_sheet(wb, ws, "default");
        // save and download
        XLSX.writeFile(wb, "VariableResults.xlsx");
    }
}
