import { Component, Input } from "@angular/core";
import { MatDialog } from "@angular/material";
import { NgParameter } from "../../formulaire/ngparam";
import { ParamCalculability } from "jalhyd";
import { DialogEditParamComputedComponent } from "../dialog-edit-param-computed/dialog-edit-param-computed.component";
import { I18nService } from "../../services/internationalisation/internationalisation.service";

@Component({
    selector: "param-computed",
    templateUrl: "./param-computed.component.html",
    styleUrls: [
        "./param-computed.component.scss"
    ]
})
export class ParamComputedComponent {

    @Input()
    public param: NgParameter;

    @Input()
    public title: string;

    /**
     * id de l'input, utilisé notamment pour les tests
     */
    public get inputId() {
        let id = "calc_input-1";
        if (this.param) {
            id = "calc_" + this.param.symbol;
        }
        return id;
    }

    constructor(
        private editInitialValueDialog: MatDialog,
        private intlService: I18nService
    ) { }

    public get isDicho() {
        return this.param.paramDefinition.calculability === ParamCalculability.DICHO;
    }

    public get infoText() {
        let text = this.intlService.localizeText("INFO_PARAMFIELD_IN_CALCULATION");
        if (this.isDicho) {
            text += " (" + this.intlService.localizeText("INFO_PARAMFIELD_IN_CALCULATION_INITIAL_VALUE")
                + ": " + this.param.getValue() + ")";
        }
        return text;
    }

    public openDialog() {
        // modification de la valeur initiale, sans avoir à remettre le mode de
        // paramètre sur "fixé"
        this.editInitialValueDialog.open(
            DialogEditParamComputedComponent,
            {
                data: {
                    param: this.param
                }
            }
        );
    }
}
