import { FormulaireElement } from "./formulaire-element";
import { FieldSet } from "./fieldset";
import { FieldsetTemplate } from "./fieldset-template";
import { StringMap } from "../stringmap";
import { FormulaireNode } from "./formulaire-node";
import { Nub } from "jalhyd";

export class FieldsetContainer extends FormulaireElement {
    private _templates: FieldsetTemplate[];

    private _localisation: StringMap;

    public title: string;

    constructor(parent: FormulaireNode) {
        super(parent);
        this._templates = [];
    }

    private addTemplate(fst: FieldsetTemplate) {
        this._templates.push(new FieldsetTemplate(fst));
    }

    public getTemplate(index: number): FieldsetTemplate {
        return this._templates[index];
    }

    private getTemplateIndex(id: string): number {
        let i = 0;
        for (const t of this._templates) {
            if (t.config["id"] === id) {
                return i;
            }
            i++;
        }
        throw new Error(`template ${id} non trouvé`);
    }

    public addFieldset(fs: FieldSet) {
        this.fieldsets.push(fs);
    }

    public moveFieldsetUp(fs: FieldSet) {
        let i = 0;
        for (const f of this.kids) {
            if (f.uid === fs.uid && i > 0) {
                const t = this.kids[i - 1];
                this.kids[i - 1] = this.kids[i];
                this.kids[i] = t;
                return;
            }
            i++;
        }
    }

    public moveFieldsetDown(fs: FieldSet) {
        let i = 0;
        for (const f of this.kids) {
            if (f.uid === fs.uid && i < this.kids.length - 1) {
                const t = this.kids[i];
                this.kids[i] = this.kids[i + 1];
                this.kids[i + 1] = t;
                return;
            }
            i++;
        }
    }

    public removeFieldset(fs: FieldSet) {
        const i = fs.indexAsKid();
        this.kids.splice(i, 1);
    }

    /**
     * crée un FieldSet à partir d'un template
     * @param templateIndex indice du template dans la liste
     * @param after insère le nouveau FieldSet après cette position, à la fin sinon
     */
    public addFromTemplate(templateIndex: number, after?: number, nub?: Nub): FieldSet {
        const templ: FieldsetTemplate = this._templates[templateIndex];

        const inst: FieldSet = templ.instantiateTemplate(this, after, nub);

        this.updateLocalisation();

        // notification de création d'un FieldSet
        this.notifyObservers({
            "action": "newFieldset",
            "fieldset": inst
        }, this);

        return inst;
    }

    public get fieldsets(): FieldSet[] {
        return this.kids as FieldSet[];
    }

    public parseConfig(json: {}, data?: {}) {
        this._confId = json["id"];
        const templates = data as any[];

        const templateNames: string[] = json["templates"];
        for (const t of templateNames) {
            for (const d of templates) {
                if (d.id === t) {
                    this.addTemplate(d);
                }
            }
        }
    }

    public updateLocalisation(loc: StringMap = this._localisation) {
        this._localisation = loc;
        if (loc !== undefined) {
            super.updateLocalisation(loc);
        }
    }
}
