import { HttpService } from "../http/http.service";
import { Injectable } from "@angular/core";
import { Observable } from "jalhyd";

/**
 * Stores app preferences
 */
@Injectable()
export class ApplicationSetupService extends Observable {

    private CONFIG_FILE_PATH = "app/config.json";

    // default builtin values
    public displayPrecision = 0.001;
    public computePrecision = 0.0001;
    public newtonMaxIterations = 50;
    private _language = "fr";
    /** themes to group calculators, for displaying on the front page */
    public themes: any[];

    public constructor(
        private httpService: HttpService
    ) {

        super();
        this.readValuesFromConfig();
    }

    public get displayDigits() {
        return -Math.log10(this.displayPrecision);
    }

    public set language(lang: string) {
        this._language = lang;
        this.notifyObservers(null);
    }

    public get language(): string {
        return this._language;
    }

    // @TODO save preferences in cookie / localStorage ?

    // read default values from config and notify observers
    private readValuesFromConfig() {
        this.httpService.httpGetRequestPromise(this.CONFIG_FILE_PATH).then((data: any) => {
            this.displayPrecision = data.params.displayPrecision;
            this.computePrecision = data.params.computePrecision;
            this.newtonMaxIterations = data.params.newtonMaxIterations;
            this.language = data.params.language;
            this.themes = data.themes;

            this.notifyObservers(null);
        });
    }
}
